/*
    <one line to give the program's name and a brief idea of what it does.>
    Copyright (C) 2013  PCMan <email>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*/


#ifndef FM_QIMAGETHUMBNAILER_H
#define FM_QIMAGETHUMBNAILER_H

#include <QObject>
#include <QQueue>
#include <QCryptographicHash>
#include "thumbnailer.h"

namespace Fm {

class QImageThumbnailer : public Thumbnailer {
Q_OBJECT

public:

  static void init();
  static void finalize();

  explicit QImageThumbnailer(FmFileInfo* file, int size);
  virtual ~QImageThumbnailer();

  virtual void start();
  virtual void cancel();

  static bool fileSupported(FmFileInfo* file);
};

class QImageThumbnailerEngine {
  
public:
  QImageThumbnailerEngine();
  virtual ~QImageThumbnailerEngine();
  
  void add(Thumbnailer* thumbnailer);
  void remove(Thumbnailer* thumbnailer);
  
public:
  static QImageThumbnailerEngine* instance() {
    return theInstance;
  }
  
  static bool fileSupported(FmFileInfo* file);
  
private:
  
  enum ThumbnailSize {
    Large,
    Small
  };
  
  QString thumbnailName(FmFileInfo* file);
  bool run(GIOSchedulerJob* job);
  static gboolean _run(GIOSchedulerJob* job, Fm::QImageThumbnailerEngine* pThis);
  static void emitFinish(QImageThumbnailerEngine* pThis);
  
  void addToBuiltInLoader(Thumbnailer* thumbnailer);
  void addToExternalLoader(Thumbnailer* thumbnailer);
  
private:
  static QImageThumbnailerEngine* theInstance;
  QString thumbnailDir;
  QCryptographicHash md5Gen;
  
  // built-in QImage-based loader
  GMutex* mutex;
  QQueue<Thumbnailer*> queue;
  Thumbnailer* current;
  GCancellable* cancellable;
  bool isRunning;
};


}

#endif // FM_QIMAGETHUMBNAILER_H
