/*
    <one line to give the program's name and a brief idea of what it does.>
    Copyright (C) 2013  PCMan <email>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*/


#ifndef FM_THUMBNAILER_H
#define FM_THUMBNAILER_H

#include <QObject>
#include <QImage>
#include <libfm/fm.h>

namespace Fm {

class ThumbnailerEngine;

class LIBFM_QT_API Thumbnailer: public QObject {

Q_OBJECT
public:

  friend class ThumbnailerEngine;

  static Thumbnailer* fromFile(FmFileInfo* file, int size);

  void start();
  
  void cancel();

  FmFileInfo* fileInfo() {
    return file_;
  }
  
  FmMimeType* mimeType() {
    return fm_file_info_get_mime_type(file_);
  }
  
  FmPath* path() {
    return fm_file_info_get_path(file_);
  }

  QImage result() {
    return result_;
  }

  int size() {
    return size_;
  }

  void setResult(QImage result) {
    result_ = result;
  }

Q_SIGNALS:
  void finished();

protected:

  explicit Thumbnailer(FmFileInfo* file, int size, ThumbnailerEngine* engine);

private:
  FmFileInfo* file_;
  QImage result_;
  int size_;
  ThumbnailerEngine* engine_;
};


class ThumbnailerEngine: public QObject {
Q_OBJECT
public:
  ThumbnailerEngine() {
  }
  virtual ~ThumbnailerEngine() {
  }

  virtual void add(Thumbnailer* thumbnailer) = 0;
  virtual void remove(Thumbnailer* thumbnailer) = 0;
  virtual bool fileSupported(FmFileInfo* file) = 0;
};

}

#endif // FM_THUMBNAILER_H
